__author__ = 'teemu kanstren'

#log into mysql? you can use create_db.sql file to create the schema
MYSQL_ENABLED = False
#log directly into elasticsearch (over the network)
ES_NW_ENABLED = False
#log to file using elasticsearch bulk format
ES_FILE_ENABLED = False
#log into a file using CSV format
CSV_ENABLED = True
#print log data to console (CSV/ES only)
PRINT_CONSOLE = False
KAFKA_ENABLED = True

#elasticsearch server host address
ES_HOST = "localhost"
#elasticsearch host server port
ES_PORT = 9200

#name of the data collection session
SESSION_NAME = "session1"
#name of the elasticsearch index to write logs into
ES_INDEX = SESSION_NAME

KAFKA_SERVER = "localhost:9092"
KAFKA_TOPIC = "pypro"

#polling interval. this is the time the poller sleeps between reading and writing values.
INTERVAL = 1

SNMP_OIDS = []
SNMP_AUTH = False
SNMP_USER = 'my_user'
SNMP_PASS = 'my_pass'
SNMP_PRIVKEY = 'my_priv_key'
SNMP_AUTH_PROTO = 'SHA'
SNMP_PRIV_PROTO = 'AES'


__author__ = 'teemu kanstren'

#http://www.mibdepot.com/cgi-bin/getmib3.cgi?i=1&n=UCD-SNMP-MIB&r=f5&f=UCD-SNMP-MIB&v=v2&t=tree

import pypro.snmp.config as config
from pypro.snmp.oid import OID
import pypro.snmp.main as main
from pypro.snmp.oids import *

config.SESSION_NAME = "session2"
config.ES_INDEX = "pypro-snmp"
config.ES_NW_ENABLED = False
config.ES_FILE_ENABLED = True
config.CSV_ENABLED = True
config.KAFKA_ENABLED = False
config.KAFKA_TOPIC = "session2"
config.KAFKA_SERVER = "192.168.2.153"
config.PRINT_CONSOLE = True
config.SNMP_AUTH = False

#example for querying whatever OID you like
config.SNMP_OIDS.append(OID('1.3.6.1.2.1.1.3.0', 'system uptime', 'public', '192.168.2.1', 161, 'router', True))
#following are examples of the buildin OID to query
#raw user space cpu time
config.SNMP_OIDS.append(UserCPUTimeRaw('public', '192.168.2.1', 161, 'router'))
#percentage of user space cpu time
config.SNMP_OIDS.append(UserCPUTimePrct('public', '192.168.2.1', 161, 'router'))
#raw system cpu time
config.SNMP_OIDS.append(SystemCPUTimeRaw('public', '192.168.2.1', 161, 'router'))
#percentage of system time
config.SNMP_OIDS.append(SystemCPUTimePrct('public', '192.168.2.1', 161, 'router'))
#raw idle cpu time
config.SNMP_OIDS.append(IdleCPUTimeRaw('public', '192.168.2.1', 161, 'router'))
#percentage of idle time
config.SNMP_OIDS.append(IdleCPUTimePrct('public', '192.168.2.1', 161, 'router'))
#total ram in system
config.SNMP_OIDS.append(RamTotal('public', '192.168.2.1', 161, 'router'))
#total ram free
config.SNMP_OIDS.append(RamFree('public', '192.168.2.1', 161, 'router'))
#available disk space, requires modifying snmp config on host
config.SNMP_OIDS.append(DiskTotal('public', '192.168.2.1', 161, 'router'))
#used disk space, requires modifying snmp.config on host
config.SNMP_OIDS.append(DiskUsed('public', '192.168.2.1', 161, 'router'))
#bytes in (network interface 1, the last number..)
config.SNMP_OIDS.append(BytesIn('public', '192.168.2.1', 161, 'router', 1))
#bytes out (network interface 1, the last number..)
config.SNMP_OIDS.append(BytesOut('public', '192.168.2.1', 161, 'router', 1))
#used memory. counted as total-available
config.SNMP_OIDS.append(RamUsed('public', '192.168.2.1', 161, 'router'))

main.run_poller()



__author__ = 'teemu kanstren'

import time
from pysnmp.entity.rfc3413.oneliner import cmdgen
from pypro.snmp import config

class SNMPPoller:
    def __init__(self, oid, snmp, loggers):
        self.oid = oid
        self.snmp = snmp
        self.loggers = loggers

    def poll(self):
        oid = self.oid
#        error_indication, error_status, error_index, var_binds = None
        if config.SNMP_AUTH:
            user = config.SNMP_USER
            password = config.SNMP_PASS
            privacy_key = config.SNMP_PRIVKEY
            auth_proto = cmdgen.usmHMACMD5AuthProtocol
            if config.SNMP_AUTH_PROTO.lower() == "SHA".lower():
                auth_proto = cmdgen.usmHMACSHAAuthProtocol
            priv_proto = cmdgen.usmAesCfb128Protocol
            if config.SNMP_PRIV_PROTO.lower() == "DES":
                priv_proto = cmdgen.usmDESPrivProtocol
            error_indication, error_status, error_index, var_binds = \
                oid.measure_auth(self.snmp, user, password, privacy_key, auth_proto, priv_proto)
#            error_indication, error_status, error_index, var_binds = self.snmp.getCmd(
#                cmdgen.UsmUserData(user, authKey=password, privKey=privacy_key,
#                                   authProtocol=auth_proto, privProtocol=priv_proto),
#                cmdgen.UdpTransportTarget((oid.ip, oid.port)), oid.oid,
#                lookupNames=True, lookupValues=True
#            )
        else:
            error_indication, error_status, error_index, var_binds = oid.measure_base(self.snmp)
#            error_indication, error_status, error_index, var_binds = self.snmp.getCmd(
#                cmdgen.CommunityData(oid.community),
#                cmdgen.UdpTransportTarget((oid.ip, oid.port)), oid.oid_id,
#                lookupNames=True, lookupValues=True
#            )

        epoch = int(time.time())
        # Check for errors and print out results
        if error_indication:
            for logger in self.loggers:
                logger.error(epoch, "failed to poll oid "+oid.oid_id+" from "+oid.target()+" : "+str(error_indication))
        elif error_status:
            for logger in self.loggers:
                logger.error(epoch, "failed to poll oid "+oid.oid_id+" from "+oid.target()+" : "+str(error_status))
        else:
            for name, val in var_binds:
                for logger in self.loggers:
                    logger.value(epoch, oid, val)
        pass



__author__ = 'teemu kanstren'

from elasticsearch import Elasticsearch
import json

import pypro.snmp.config as config
from pypro import utils

class ESNetLogger:
    indices = {}

    def __init__(self):
        self.event_id = 1

        self.es = Elasticsearch([config.ES_HOST+':'+str(config.ES_PORT)],
                                sniff_on_start=False, sniff_on_connection_fail=False, sniffer_timeout=60)
        for oid in config.SNMP_OIDS:
            self.indices[oid._name()] = 0

    def close(self):
        #todo: check is es needs closing
        pass

    def start(self, epoch):
        epoch *= 1000
        body = '{"time" : ' + str(epoch) + ', "session_info" : "start"}'
        reply = self.es.index(index=config.ES_INDEX, doc_type="event", id='event_' + str(self.event_id), body=body)
        self.event_id += 1
        if config.PRINT_CONSOLE: print(reply)

    def stop(self, epoch):
        epoch *= 1000
        body = '{"time" : ' + str(epoch) + ', "session_info" : "stop"}'
        reply = self.es.index(index=config.ES_INDEX, doc_type="event", id='event_' + str(self.event_id), body=body)
        self.event_id += 1
        if config.PRINT_CONSOLE: print(reply)

    def value(self, epoch, oid, value):
        epoch *= 1000
        name = oid._name()
        index = self.indices[name]
        index += 1
        self.indices[name] = index
        str_value = str(value)
        if not oid.numeric or not utils.is_number(str_value):
            str_value = json.dumps(str_value)
        body = '{"time" : ' + str(epoch) + ', "target" : "' + str(oid.target()) + '", ' + \
               '"target_name" : "' + str(oid.target_name) + '", "oid" : "' + str(oid.oid_id) + '", ' + \
               '"oid_name" : "' + str(oid.oid_name) + '", "value" : ' + str_value + '}'
        reply = self.es.index(index=config.ES_INDEX, doc_type=name, id=name + '_' + str(index), body=body)
        if config.PRINT_CONSOLE: print(reply)

    def error(self, epoch, description):
        body = '{"time" : ' + str(epoch) + ', "error" : "' + description + '"}'
        reply = self.es.index(index=config.ES_INDEX, doc_type="event", id='event_' + str(self.event_id), body=body)
        self.event_id += 1
        if config.PRINT_CONSOLE: print(reply)



__author__ = 'teemu kanstren'

import time

from pysnmp.entity.rfc3413.oneliner import cmdgen

import pypro.snmp.config as config
from pypro.snmp.loggers.es_network_logger import ESNetLogger
from pypro.snmp.loggers.es_file_logger import ESFileLogger
from pypro.snmp.loggers.csv_logger import CSVFileLogger
from pypro.snmp.loggers.kafka_logger import KafkaLogger
from pypro.snmp.snmp_poller import SNMPPoller

#resource OID's on linux: http://www.debianadmin.com/linux-snmp-oids-for-cpumemory-and-disk-statistics.html
#also http://kaivanov.blogspot.fi/2012/02/linux-snmp-oids-for-cpumemory-and-disk.html
#snmpwalk = get the whole subtree from given node
#snmpgetnext = get next for given value in tree. looping gives walk.

def init():
#    global cmd_gen
    global loggers
#    global oids
    snmp = cmdgen.CommandGenerator()
    loggers = []
    oids = []
    if (config.ES_FILE_ENABLED): loggers.append(ESFileLogger())
    if (config.ES_NW_ENABLED): loggers.append(ESNetLogger())
    if (config.CSV_ENABLED): loggers.append(CSVFileLogger())
    if (config.KAFKA_ENABLED): loggers.append(KafkaLogger())

    global pollers
    pollers = []
    for oid in config.SNMP_OIDS:
        pollers.append(SNMPPoller(oid, snmp, loggers))
    epoch = int(time.time())
    for logger in loggers:
        logger.start(epoch)

def poll():
    for poller in pollers:
        poller.poll()

def run_poller():
#    time_this("init:", init)
    init()
    while True:
        poll()
        time.sleep(config.INTERVAL)
    shutdown()

def shutdown():
    epoch = int(time.time())
    for logger in loggers:
        logger.stop(epoch)

if __name__ == "__main__":
    print ("start at:"+str(int(time.time())))
    run_poller()



__author__ = 'teemu kanstren'

import time

from pypro.snmp import config
from pypro.head_builder import HeadBuilder
from pypro import utils

class KafkaLogger:
    indices = {}

    def __init__(self):
        from kafka import SimpleProducer, KafkaClient
        from kafka.common import LeaderNotAvailableError
        self.kafka_client = KafkaClient(config.KAFKA_SERVER)
        self.kafka = SimpleProducer(self.kafka_client)
        self.event_id = 1

        for oid in config.SNMP_OIDS:
            self.indices[oid._name()] = 0

        self.head = HeadBuilder("index", "doc_type", "id", config.ES_INDEX)
        try:
            self.kafka.send_messages(config.KAFKA_TOPIC, b"creating topic")
        except LeaderNotAvailableError:
            time.sleep(1)

    def close(self):
        self.kafka.stop(0)
        self.kafka_client.close()

    def start(self, epoch):
#        epoch *= 1000 #this converts it into milliseconds
        head = self.head.create('event', 'event_' + str(self.event_id))
        body = '{"start_time" : ' + str(epoch) + ', "session_info" : "start"}'
        msg = '{"header": '+ head + ', "body":'+ body+'}'
        self.kafka.send_messages(config.KAFKA_TOPIC, msg.encode("utf8"))
        self.event_id += 1
        if config.PRINT_CONSOLE: print(msg)

    def stop(self, epoch):
#        epoch *= 1000 #this converts it into milliseconds
        head = self.head.create('event', 'event_' + str(self.event_id))
        body = '{"time" : ' + str(epoch) + ', "session_info" : "stop"}'
        msg = '{"header": '+ head + ', "body":'+ body+'}'
        self.kafka.send_messages(config.KAFKA_TOPIC, msg.encode("utf8"))
        self.event_id += 1
        if config.PRINT_CONSOLE: print(msg)

    def value(self, epoch, oid, value):
        name = oid._name()
        index = self.indices[name]
        index += 1
        self.indices[name] = index
        is_str = False
        str_value = str(value)
        if not oid.numeric or not utils.is_number(str_value):
            is_str = True
#        head = self.head.create(name, name+'_' + str(index))
        head = self.head.create("measurement", name+'_' + str(index))
        body = '{"time" : ' + str(epoch) + ', "target" : "' + str(oid.target()) + '", ' + \
               '"target_name" : "' + str(oid.target_name) + '", "oid" : "' + str(oid.oid_id) + '", ' + \
               '"oid_name" : "' + str(oid.oid_name)
        if is_str:
            body += '", "str_value" : "' + str_value + '"}'
        else:
            body += '", "value" : ' + str_value + '}'
        msg = '{"header": '+ head + ', "body":'+ body+'}'
        self.kafka.send_messages(config.KAFKA_TOPIC, msg.encode("utf8"))
        if config.PRINT_CONSOLE: print(msg)

    def error(self, epoch, description):
#        epoch *= 1000 #this converts it into milliseconds
        head = self.head.create('event', 'event_' + str(self.event_id))
        body = '{"time" : ' + str(epoch) + ', "error" : "'+description+'"}'
        msg = '{"header": '+ head + ', "body":'+ body+'}'
        self.kafka.send_messages(config.KAFKA_TOPIC, msg.encode("utf8"))
        self.event_id += 1
        if config.PRINT_CONSOLE: print(msg)




__author__ = 'teemu kanstren'

import os

log_dir = "logs/"
session_log = log_dir+"session-info"
cpu_sys_log = log_dir+"cpu-log-sys"
cpu_proc_log = log_dir+"cpu-log-proc"
mem_sys_log = log_dir+"mem-log-sys"
mem_proc_log = log_dir+"mem-log-proc"
io_sys_log = log_dir+"io-log-sys"
event_log = log_dir+"event-log"
proc_info_log = log_dir+"proc-log-info"

def check_dir():
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def isnumber(x):
    "Is x a number? We say it is if it has an __int__ method."
    return hasattr(x, '__int__')


__author__ = 'teemu kanstren'

import pypro.utils as utils
import pypro.snmp.config as config
from pypro.head_builder import HeadBuilder


class ESFileLogger:
    files = {}
    indices = {}

    def __init__(self):
        oids = config.SNMP_OIDS
        utils.check_dir()

        self.head = HeadBuilder("_index", "_type", "_id", config.ES_INDEX)

        self.event_id = 1
        self.event_log = open(utils.event_log + ".es", "w", encoding="utf-8")

        for oid in oids:
            file_name = utils.log_dir + "/" + oid._name() + "_log.es"
            log = open(file_name, "w", encoding="utf-8")
            self.files[oid.oid_id] = log
            self.indices[oid._name()] = 0

    def close(self):
        for file in self.files:
            file.close()
        self.event_log.close()

    def start(self, epoch):
        epoch *= 1000 #this converts it into milliseconds
        head = '{"index" : ' + self.head.create('event', 'event_' + str(self.event_id)) + '}\n'
        body = '{"time" : ' + str(epoch) + ', "session_info" : "start"}'
        line = head + body
        self.event_log.write(line + "\n")
        self.event_log.flush()
        self.event_id += 1
        if config.PRINT_CONSOLE: print(line)

    def stop(self, epoch):
        epoch *= 1000
        head = '{"index" : ' + self.head.create('event', 'event_' + str(self.event_id)) + '}\n'
        body = '{"time" : ' + str(epoch) + ', "session_info" : "stop"}'
        line = head + body
        self.event_log.write(line + "\n")
        self.event_log.flush()
        self.event_id += 1
        if config.PRINT_CONSOLE: print(line)

    def value(self, epoch, oid, value):
        epoch *= 1000
        name = oid._name()
        index = self.indices[name]
        index += 1
        self.indices[name] = index
        str_value = str(value)
        if not oid.numeric or not utils.is_number(str_value):
            str_value = '"'+str_value+'"'
#        else:
#            if not utils.is_number(str_value):
#                self.error(epoch, "Numeric OID "+oid.oid+" produced non-numeric value:"+str_value)
#                return
        head = '{"index" : ' + self.head.create(name, name + '_' + str(index)) + '}\n'
        body = '{"time" : ' + str(epoch) + ', "target" : "' + str(oid.target()) + '", ' + \
               '"target_name" : "' + str(oid.target_name) + '", "oid" : "' + str(oid.oid_id) + '", ' + \
               '"oid_name" : "' + str(oid.oid_name) + '", "value" : ' + str_value + '}'
        line = head + body
        log = self.files[oid.oid_id]
        log.write(line + "\n")
        log.flush()
        if config.PRINT_CONSOLE: print(line)

    def error(self, epoch, description):
        epoch *= 1000
        head = '{"index" : ' + self.head.create('event', 'event_' + str(self.event_id)) + '}\n'
        body = '{"time" : ' + str(epoch) + ', "error" : "' + description + '"}'
        line = head + body
        self.event_log.write(line + "\n")
        self.event_log.flush()
        self.event_id += 1
        if config.PRINT_CONSOLE: print(line)



__author__ = 'teemu kanstren'

import pypro.utils as utils
import pypro.snmp.config as config

class CSVFileLogger:
    files = {}

    def __init__(self):
        oids = config.SNMP_OIDS
        utils.check_dir()

        self.event_log = open(utils.event_log+".csv", "w", encoding="utf-8")
        event_header = "time;type;description"
        self.event_log.write(event_header + "\n")
        self.event_log.flush()

        for oid in oids:
            file_name = utils.log_dir + "/" + oid.oid_name.replace(' ', '_') + "_log.csv"
            log = open(file_name, "w", encoding="utf-8")
            header = "time;target;target_name;oid;oid_name;value"
            log.write(header + "\n")
            log.flush()
            self.files[oid.oid_id] = log

    def close(self):
        for file in self.files:
            file.close()
        self.event_log.close()

    def start(self, epoch):
        line = str(epoch) + ";info;session started ("+config.SESSION_NAME+")"
        self.event_log.write(line + "\n")
        self.event_log.flush()
        if config.PRINT_CONSOLE: print(line)

    def stop(self, epoch):
        line = str(epoch) + ";info;session stopped ("+config.SESSION_NAME+")"
        self.event_log.write(line + "\n")
        self.event_log.flush()
        if config.PRINT_CONSOLE: print(line)
        self.close()

    def value(self, epoch, oid, value):
        line = str(epoch) + ";" + oid.target() + ";" + oid.target_name + ";" + str(oid.oid_id) + ";" + str(oid.oid_name) + ";" + str(value)
        log = self.files[oid.oid_id]
        log.write(line + "\n")
        log.flush()
        if config.PRINT_CONSOLE: print(line)

    def error(self, epoch, description):
        line = str(epoch) + ";error;" + description
        self.event_log.write(line + "\n")
        self.event_log.flush()
        if config.PRINT_CONSOLE: print(line)



__author__ = 'teemu kanstren'

class HeadBuilder:
    def __init__(self, index_tag, type_tag, id_tag, index_value):
        self.index_tag = index_tag
        self.type_tag = type_tag
        self.id_tag = id_tag
        self.index_value = index_value

    def create(self, type, id):
        line = '{"'+self.index_tag+'" : "'+self.index_value+'", "'+self.type_tag+'" : "'+type+'", "'+self.id_tag+'" : "'+id+'"}'
        return  line



__author__ = 'teemu kanstren'

from setuptools import setup

setup(name='pypro-snmp',
      version='0.2.0',
      description='System resource probes',
      url='https://github.com/mukatee/pypro',
      author='Teemu Kanstren',
      author_email='tkanstren@gmail.com',
      license='MIT',
      packages=['pypro', 'pypro.snmp', 'pypro.snmp.loggers'],
      zip_safe=True,
      install_requires=[
          'psutil', 'elasticsearch', 'kafka-python', 'pysnmp'
      ],
      classifiers=['License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3.4',
                   'Development Status :: 4 - Beta',
                   'Topic :: System :: Networking :: Monitoring'],
      keywords='monitoring',
     )



__author__ = 'teemu kanstren'

from pysnmp.entity.rfc3413.oneliner import cmdgen
from pypro.snmp.oid import OID


class UserCPUTimeRaw(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '1.3.6.1.4.1.2021.11.50.0', 'user cpu time', community, ip, port, target_name, True)


class UserCPUTimePrct(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '1.3.6.1.4.1.2021.11.9.0', 'percentage user cpu time', community, ip, port, target_name,
                     True)


class SystemCPUTimeRaw(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '1.3.6.1.4.1.2021.11.52.0', 'system cpu time', community, ip, port, target_name, True)


class SystemCPUTimePrct(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '1.3.6.1.4.1.2021.11.10.0', 'percentage system cpu time', community, ip, port, target_name,
                     True)


class IdleCPUTimeRaw(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.11.53.0', 'idle cpu time', community, ip, port, target_name, True)


class IdleCPUTimePrct(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.11.11.0', 'percentage idle cpu time', community, ip, port, target_name,
                     True)


class RamTotal(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.4.5.0', 'total RAM', community, ip, port, target_name, True)


class RamFree(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.4.11.0', 'free RAM', community, ip, port, target_name, True)


# total available disk space, requires modifying snmp config on host
class DiskTotal(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.9.1.7.1', 'total disk space', community, ip, port, target_name, True)


# used disk space, requires modifying snmp.config on host
class DiskUsed(OID):
    def __init__(self, community, ip, port, target_name):
        OID.__init__(self, '.1.3.6.1.4.1.2021.9.1.8.1', 'used disk space', community, ip, port, target_name, True)


# bytes in for a network interface. parameter "n" defines interface number
class BytesIn(OID):
    def __init__(self, community, ip, port, target_name, n):
        OID.__init__(self, '.1.3.6.1.2.1.2.2.1.10.1.' + str(n), 'nw bytes in if ' + str(n), community, ip, port, target_name, True)


# bytes out for a network interface. parameter "n" defines interface number
class BytesOut(OID):
    def __init__(self, community, ip, port, target_name, n):
        OID.__init__(self, '.1.3.6.1.2.1.2.2.1.16.1.' + str(n), 'nw bytes out if ' + str(n), community, ip, port, target_name, True)

# discarded incoming packets for a network interface. parameter "n" defines interface number
class DiscardedIn(OID):
    def __init__(self, community, ip, port, target_name, n):
        OID.__init__(self, '1.3.6.1.2.1.2.2.1.13.' + str(n), 'discarded packets in if ' + str(n), community, ip, port, target_name, True)

# discarded outgoing packets for a network interface. parameter "n" defines interface number
class DiscardedOut(OID):
    def __init__(self, community, ip, port, target_name, n):
        OID.__init__(self, '1.3.6.1.2.1.2.2.1.19.' + str(n), 'discarded packets out if ' + str(n), community, ip, port, target_name, True)

#ram used. a measure derived from reading two oid values, total ram in system and free ram in system.
class RamUsed:
    def __init__(self, community, ip, port, target_name):
        # sometimes the OID number sequence starts with "." which works to read it
        # but pysnmp returns the name without the first "." so have to that here as well for later comparison to work
        self.oid_total_id = '1.3.6.1.4.1.2021.4.5.0'
        self.oid_free_id = '1.3.6.1.4.1.2021.4.11.0'
        self.oid_name = 'used ram'
        self.community = community
        self.ip = ip
        self.port = port
        self.target_name = target_name
        self.numeric = True
        self.oid_id = 'used mem'

    def target(self):
        return self.ip + ":" + str(self.port)

    def _name(self):
        return self.oid_name.replace(' ', '_')

    #for authenticated measurement polls
    def measure_auth(self, snmp, user, password, privacy_key, auth_proto, priv_proto):
        errorIndication, errorStatus, errorIndex, var_binds = snmp.getCmd(
            cmdgen.UsmUserData(user, authKey=password, privKey=privacy_key,
                               authProtocol=auth_proto, privProtocol=priv_proto),
            cmdgen.UdpTransportTarget((self.ip, self.port)),
            self.oid_total_id, #list of OID's to poll is here
            self.oid_free_id,
            lookupNames=True, lookupValues=True
        )
        return self.calc_used(errorIndication, errorStatus, errorIndex, var_binds)

    #for basic measurements with not authentication
    def measure_base(self, snmp):
        errorIndication, errorStatus, errorIndex, var_binds = snmp.getCmd(
            cmdgen.CommunityData(self.community),
            cmdgen.UdpTransportTarget((self.ip, self.port)),
            self.oid_total_id, #list of OID's to poll is here
            self.oid_free_id)
        return self.calc_used(errorIndication, errorStatus, errorIndex, var_binds)

    def calc_used(self, errorIndication, errorStatus, errorIndex, var_binds):
        if errorIndication or errorStatus:
            return (errorIndication, errorStatus, errorIndex, [])
        for name, val in var_binds:
            #print('%s = %s' % (name.prettyPrint(), val.prettyPrint()))
            pretty = str(name)
            if pretty == self.oid_total_id:
                total = val.prettyPrint() #here we pick up the total ram count in the system (oid 1)
            if pretty == self.oid_free_id:
                free = val.prettyPrint() #and here we pick up the free ram count in the system (oid 2)
        value = int(total) - int(free) #and calculate the derived measure (free ram count)
        return (errorIndication, errorStatus, errorIndex, [(self.oid_id, value)]) #finally return it in similar for to pysnmp



__author__ = 'teemu kanstren'

from pysnmp.entity.rfc3413.oneliner import cmdgen

class OID:
    def __init__(self, oid_id, oid_name, community, ip, port, target_name, numeric):
        self.oid_id = oid_id
        self.oid_name = oid_name
        self.community = community
        self.ip = ip
        self.port = port
        self.target_name = target_name
        self.numeric = numeric

    def target(self):
        return self.ip+":"+str(self.port)

    def _name(self):
        return self.oid_name.replace(' ', '_')

    def measure_auth(self, snmp, user, password, privacy_key, auth_proto, priv_proto):
        return snmp.getCmd(
            cmdgen.UsmUserData(user, authKey=password, privKey=privacy_key,
                               authProtocol=auth_proto, privProtocol=priv_proto),
            cmdgen.UdpTransportTarget((self.ip, self.port)), self.oid_id,
            lookupNames=True, lookupValues=True
        )

    def measure_base(self, snmp):
        errorIndication, errorStatus, errorIndex, var_binds = snmp.getCmd(
            cmdgen.CommunityData(self.community),
            cmdgen.UdpTransportTarget((self.ip, self.port)), self.oid_id,
            lookupNames=True, lookupValues=True
        )
        values = []
        for name, val in var_binds:
#            print('%s = %s' % (name.prettyPrint(), val.prettyPrint()))
            values.append((name.prettyPrint(), val.prettyPrint()))
        return (errorIndication, errorStatus, errorIndex, values)
